EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'DebianPackages_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'DebianPackage_Id',
    /* Description_Text        */ 'Creates or updates a package in a Debian feed.'
GO

IF OBJECT_ID('[DebianPackages_CreateOrUpdatePackage]') IS NOT NULL DROP PROCEDURE [DebianPackages_CreateOrUpdatePackage]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [DebianPackages_CreateOrUpdatePackage]
(
	@Feed_Id INT,
	@Component_Name VARCHAR(255),
	@Package_Name VARCHAR(255),
	@Architecture_Name VARCHAR(255),
	@Version_Text VARCHAR(255),
	@PackageMetadata_Bytes VARBINARY(MAX) = NULL,
	@Published_Date DATETIME = NULL,
	@PackageSha256_Bytes BINARY(32) = NULL,
	@PackageSha512_Bytes BINARY(64) = NULL,
	@Cached_Indicator YNINDICATOR = NULL,
	@Package_Size BIGINT = NULL,
	@Description_Text NVARCHAR(MAX) = NULL,
	@Download_Count INT = NULL,
	@DebianPackage_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

DECLARE @Package_Id VARCHAR(MAX) = @Component_Name + '/' + @Package_Name + ':' + @Architecture_Name

BEGIN TRY
	BEGIN TRANSACTION

		SET @DebianPackage_Id = (SELECT [DebianPackage_Id]
		                           FROM [DebianPackages]
		                           WITH (TABLOCKX)
		                          WHERE [Feed_Id] = @Feed_Id
		                            AND [Component_Name] = @Component_Name
		                            AND [Package_Name] = @Package_Name
		                            AND [Architecture_Name] = @Architecture_Name
		                            AND [Version_Text] = @Version_Text)

		IF @DebianPackage_Id IS NULL BEGIN

			INSERT INTO [DebianPackages] (
				[Feed_Id],
				[Component_Name],
				[Package_Name],
				[Architecture_Name],
				[Version_Text],
				[PackageMetadata_Bytes],
				[Published_Date],
				[PackageSha256_Bytes],
				[PackageSha512_Bytes],
				[Cached_Indicator],
				[Package_Size],
				[Description_Text],
				[Download_Count]
			) VALUES (
				@Feed_Id,
				@Component_Name,
				@Package_Name,
				@Architecture_Name,
				@Version_Text,
				@PackageMetadata_Bytes,
				COALESCE(@Published_Date, GETUTCDATE()),
				@PackageSha256_Bytes,
				@PackageSha512_Bytes,
				COALESCE(@Cached_Indicator, 'N'),
				@Package_Size,
				@Description_Text,
				COALESCE(@Download_Count, 0)
			)

			SET @DebianPackage_Id = SCOPE_IDENTITY()

			EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Name', @Package_Id, 'Package_Version', @Version_Text

		END ELSE BEGIN

			UPDATE [DebianPackages]
			   SET [PackageMetadata_Bytes] = COALESCE(@PackageMetadata_Bytes, [PackageMetadata_Bytes]),
			       [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
			       [PackageSha256_Bytes] = COALESCE(@PackageSha256_Bytes, [PackageSha256_Bytes]),
			       [PackageSha512_Bytes] = COALESCE(@PackageSha512_Bytes, [PackageSha512_Bytes]),
			       [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
			       [Package_Size] = COALESCE(@Package_Size, [Package_Size]),
			       [Description_Text] = COALESCE(@Description_Text, [Description_Text]),
			       [Download_Count] = COALESCE(@Download_Count, [Download_Count])
			 WHERE [DebianPackage_Id] = @DebianPackage_Id

			EXEC [Events_RaiseEvent] 'PKGMDF', @Feed_Id, 'Package_Name', @Package_Id, 'Package_Version', @Version_Text

		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DebianPackages_CreateOrUpdatePackage] TO [ProGetUser_Role]
GO
